@@ -1,18 +1,15 @@
-# $Id: MergeVideoAudio.pm,v 1.7 2005/12/26 13:57:47 joern Exp $
+# $Id: MergeVideoAudio.pm,v 1.1 2002/06/23 21:46:40 joern Exp $
 
 #-----------------------------------------------------------------------
-# Copyright (C) 2001-2003 Jrn Reder <joern AT zyn.de>.
-# All Rights Reserved. See file COPYRIGHT for details.
+# Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
 # 
 # This program is part of Video::DVDRip, which is free software; you can
 # redistribute it and/or modify it under the same terms as Perl itself.
 #-----------------------------------------------------------------------
 
 package Video::DVDRip::Cluster::Job::MergeVideoAudio;
-use Locale::TextDomain qw (video.dvdrip);
-use Video::DVDRip::FixLocaleTextDomainUTF8;
 
-use base Video::DVDRip::Job::MergeAudio;
+use base Video::DVDRip::Cluster::Job;
 
 use Carp;
 use strict;
@@ -30,29 +27,10 @@ sub type {
 sub info {
 	my $self = shift;
 
-	return "merge video chunks psu ".$self->psu if $self->title->is_ogg;
 	return "multiplex video and audio psu ".$self->psu;
 }
 
-sub init {
-	my $self = shift;
-	 
-	my $project  = $self->project;
-	$project->set_assigned_job ( $self );
-	$self->SUPER::init;
-	$project->set_assigned_job ( undef );
-	 
-	$self->set_progress_max (
-		$self->title
-		     ->program_stream_units
-		     ->[$self->psu]
-		     ->frames
-	);
-	
-	1;
-}
-
-sub command {
+sub start {
 	my $self = shift;
 
 	my $project  = $self->project;
@@ -63,7 +41,32 @@ sub command {
 	my $command = $title->get_merge_video_audio_command;
 	$project->set_assigned_job ( undef );
 
-	return $command;
+	$self->set_progress_frames_cnt ($title->frames);
+
+	my $successful_finished = 0;
+	my $first = 1;
+	$self->popen (
+		command      => $command,
+		cb_line_read => sub {
+			my ($line) = @_;
+			if ( $line =~ /\(\d+-(\d+)\)/ ) {
+				$self->set_progress_start_time(time) if $first;
+				$self->set_progress_frames ($1);
+				$first = 0;
+			} elsif ( $line =~ /DVDRIP_SUCCESS/ ) {
+				$successful_finished = 1;
+			}
+		},
+		cb_finished  => sub {
+			if ( $successful_finished ) {
+				$self->commit_job;
+			} else {
+				$self->abort_job;
+			}
+		},
+	);
+
+	1;
 }
 
 1;
