@@ -1,4 +1,4 @@
-# $Id: TranscodeAudio.pm,v 1.2 2002/03/17 18:53:25 joern Exp $
+# $Id: TranscodeAudio.pm,v 1.3 2002/06/23 21:43:35 joern Exp $
 
 #-----------------------------------------------------------------------
 # Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
@@ -14,6 +14,12 @@ use base Video::DVDRip::Cluster::Job;
 use Carp;
 use strict;
 
+sub psu				{ shift->{psu}				}
+sub chunk_cnt			{ shift->{chunk_cnt}			}
+
+sub set_psu			{ shift->{psu}			= $_[1]	}
+sub set_chunk_cnt		{ shift->{chunk_cnt}		= $_[1]	}
+
 sub type {
 	return "transcode audio";
 }
@@ -49,7 +55,7 @@ sub start {
 		command      => $command,
 		cb_line_read => sub {
 			my ($line) = @_;
-			if ( $line =~ /split.*?frames.*?-c\s+\d+-(\d+)/ ) {
+			if ( $line =~ /split.*?mapped.*?-c\s+\d+-(\d+)/ ) {
 				$self->set_progress_frames_cnt ($1);
 				$frames_cnt = $1;
 				$self->set_progress_start_time(time);
