@@ -1,18 +1,15 @@
-# $Id: TranscodeVideo.pm,v 1.7 2005/12/26 13:57:47 joern Exp $
+# $Id: TranscodeVideo.pm,v 1.1 2002/06/23 21:46:40 joern Exp $
 
 #-----------------------------------------------------------------------
-# Copyright (C) 2001-2003 Jrn Reder <joern AT zyn.de>.
-# All Rights Reserved. See file COPYRIGHT for details.
+# Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
 # 
 # This program is part of Video::DVDRip, which is free software; you can
 # redistribute it and/or modify it under the same terms as Perl itself.
 #-----------------------------------------------------------------------
 
 package Video::DVDRip::Cluster::Job::TranscodeVideo;
-use Locale::TextDomain qw (video.dvdrip);
-use Video::DVDRip::FixLocaleTextDomainUTF8;
 
-use base Video::DVDRip::Job::TranscodeVideo;
+use base Video::DVDRip::Cluster::Job;
 
 use Carp;
 use strict;
@@ -32,28 +29,16 @@ sub type {
 }
 
 sub info {
-	my $self = shift; $self->trace_in;
+	my $self = shift;
 
-	return  "transcode video".
+	return  "transcode".
 		" chunk ".$self->chunk."/".$self->chunk_cnt.
 		", pass ".$self->pass.
 		", psu ".$self->psu;
 }
 
-sub init {
-	my $self = shift; $self->trace_in;
-	 
-	$self->project->set_assigned_job ( $self );
-	$self->SUPER::init;
-	$self->project->set_assigned_job ( undef );
-	 
-	$self->set_progress_max ($self->title->frames_per_chunk);
-	
-	1;
-}
-
-sub command {
-	my $self = shift; $self->trace_in;
+sub start {
+	my $self = shift;
 
 	my $project = $self->project;
 	my $title   = $project->title;
@@ -63,11 +48,42 @@ sub command {
 	my $command = $title->get_transcode_command ( pass => $self->pass );
 	$project->set_assigned_job ( undef );
 
-	return $command;
+	# start command
+	my $frames_cnt;
+	my $successful_finished = 0;
+
+	$self->set_progress_frames (0);
+
+	$self->popen (
+		command      => $command,
+		cb_line_read => sub {
+			my ($line) = @_;
+			if ( $line =~ /split.*?mapped.*?-c\s+\d+-(\d+)/ ) {
+				$self->set_progress_frames_cnt ($1);
+				$frames_cnt = $1;
+				$self->set_progress_start_time(time);
+
+			} elsif ( $line =~ /\[\d{6}-(\d+)\]/ ) {
+				$self->set_progress_frames($1);
+
+			} elsif ( $line =~ /DVDRIP_SUCCESS/ ) {
+				$successful_finished = 1;
+			}
+		},
+		cb_finished  => sub {
+			if ( $successful_finished ) {
+				$self->commit_job;
+			} else {
+				$self->abort_job;
+			}
+		},
+	);
+	
+	1;
 }
 
 sub commit {
-	my $self = shift; $self->trace_in;
+	my $self = shift;
 
 	my $node     = $self->node;
 	my $project  = $self->project;
@@ -76,9 +92,12 @@ sub commit {
 
 	$title->set_frames_finished (
 		$title->frames_finished +
-		$self->progress_max
+		$self->progress_frames_cnt
 	);
 
+	# job is finished
+	$self->set_state ('finished');
+
 	1;
 }
 
