@@ -1,4 +1,4 @@
-# $Id: Title.pm,v 1.25 2002/05/28 20:55:09 joern Exp $
+# $Id: Title.pm,v 1.28 2002/07/06 08:03:37 joern Exp $
 
 #-----------------------------------------------------------------------
 # Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
@@ -97,43 +97,44 @@ sub avi_file {					# transcode output file
 	);
 }
 
-sub audio_avi_file {				# audio only file
+sub audio_psu_file {				# audio only file
 	my $self = shift;
 
 	my $job = $self->project->assigned_job or croak "No job assigned";
 	
 	return sprintf (
-		"%s/%03d/audio-only/%s-%03d-audio.avi",
+		"%s/%03d/audio-psu-%02d/%s-%03d-audio-psu-%02d.avi",
 		$self->project->final_avi_dir,
 		$self->nr,
+		$job->psu,
 		$self->project->name,
 		$self->nr,
+		$job->psu,
+	);
+}
+
+sub audio_video_psu_dir {
+	my $self = shift;
+
+	return sprintf (
+		"%s/%03d/audio-video-psu",
+		$self->project->final_avi_dir,
+		$self->nr,
 	);
 }
 
-sub merged_chunks_avi_dir {			# directory for merged chunks
+
+sub audio_video_psu_file {				# audio only file
 	my $self = shift;
 
 	my $job = $self->project->assigned_job or croak "No job assigned";
 	
 	return sprintf (
-		"%s/%03d/chunks-merged",
-		$self->project->final_avi_dir,
-		$self->nr,
-	);
-
-}
-
-sub merged_chunks_avi_file {			# merged chunks of a PSU
-	my $self = shift;
-
-	my $job = $self->project->assigned_job or croak "No job assigned";
-	
-	return sprintf (
-		"%s/%s-%03d-merged.avi",
-		$self->merged_chunks_avi_dir,
+		"%s/%s-%03d-av-psu-%02d.avi",
+		$self->audio_video_psu_dir,
 		$self->project->name,
 		$self->nr,
+		$job->psu,
 	);
 }
 
@@ -166,6 +167,9 @@ sub get_transcode_command {
 
 	my $command   = $self->SUPER::get_transcode_command (@_);
 
+	# remove DVDRIP_SUCCESS
+	$command =~ s/&&\s+echo\s+DVDRIP_SUCCESS//;
+
 	# no audio options
 	$command =~ s/\s-[baN]\s+[^\s]+//;
 
@@ -178,11 +182,11 @@ sub get_transcode_command {
 	# add -S and -W options for chunk selection
 	$command .= " -S $psu -W $chunk,$chunk_cnt,$nav_file";
 
+	$command =~ s/-M 2//;
+
 	# add directory creation code
 	my $avi_dir = dirname $self->avi_file;
-	$command = "mkdir -m 0775 -p '$avi_dir'; $command";
-
-	$command .= " && echo DVDRIP_SUCCESS";
+	$command = "mkdir -m 0775 -p '$avi_dir' && $command";
 
 	# add node specific transcode options, if configured
 	$command = $self->combine_command_options (
@@ -191,6 +195,9 @@ sub get_transcode_command {
 		options  => $job->node->tc_options
 	) if $job->node->tc_options =~ /\S/;
 
+	# add DVDRIP_SUCCESS
+	$command .= " && echo DVDRIP_SUCCESS";
+
 	return $command;
 }
 
@@ -199,20 +206,34 @@ sub get_transcode_audio_command {
 	
 	my $job = $self->project->assigned_job or croak "No job assigned";
 
-	my $audio_avi_file = $self->audio_avi_file;
-	my $audio_avi_dir  = dirname($audio_avi_file);
+	my $audio_psu_file = $self->audio_psu_file;
+	my $audio_psu_dir  = dirname($self->audio_psu_file);
 
 	my $nice;
 	$nice = "/usr/bin/nice -n ".$self->tc_nice." "
 		if $self->tc_nice =~ /\S/;
 
 	my $command =
-		"mkdir -m 0775 -p '$audio_avi_dir' && ".
+		"mkdir -m 0775 -p '$audio_psu_dir' && ".
 		$nice.
 		"transcode -i ".$self->vob_dir.
 		" -x null,vob -g 0x0 -y raw -u 50".
 		" -a ".$self->audio_channel.
-		" -o ".$audio_avi_file;
+		" -o ".$audio_psu_file.
+		" -S ".$job->psu.
+		" -W ".$job->chunk_cnt.",".$job->chunk_cnt.",".
+		       $self->vob_nav_file;
+
+	if ( $self->video_mode eq 'ntsc' ) {
+#		$command .= " -M 2 ";
+	}
+
+	if ( $self->tc_video_framerate ) {
+		my $fr = $self->tc_video_framerate;
+		$fr = "24,1" if $fr == 23.976;
+		$fr = "30,4" if $fr == 29.97;
+		$command .= " -f $fr";
+	}
 
 	if ( $self->tc_ac3_passthrough ) {
 		$command .= " -A -N ".
@@ -234,39 +255,17 @@ sub get_transcode_audio_command {
 	return $command;
 }
 
-sub get_merge_chunks_command {
+sub get_merge_video_audio_command {
 	my $self = shift;
 	
 	my $job = $self->project->assigned_job or croak "No job assigned";
 
-	my $merged_chunks_avi_file = $self->merged_chunks_avi_file;
-	my $merged_chunks_avi_dir  = dirname ( $merged_chunks_avi_file );
-
-	my $chunks_mask = sprintf (
-		"%s/%03d/chunks-psu-??/*",
-		$self->project->final_avi_dir,
-		$self->nr
-	);
-
-	my $command = 
-		"mkdir -m 0775 -p '$merged_chunks_avi_dir'; ".
-		"avimerge -o '$merged_chunks_avi_file' -i $chunks_mask";
-
-	$command .= " && echo DVDRIP_SUCCESS";
-	
-	$command .= " && rm $chunks_mask" if $self->with_cleanup;
-
-	return $command;
-}
-
-sub get_merge_audio_command {
-	my $self = shift;
-	
-	my $job = $self->project->assigned_job or croak "No job assigned";
-
-	my $target_avi_file        = $self->target_avi_file;
-	my $target_avi_dir         = dirname ( $target_avi_file );
-	my $audio_avi_file         = $self->audio_avi_file;
+	my $avi_chunks_dir       = $self->avi_chunks_dir;
+	my $audio_video_psu_file = $self->audio_video_psu_file;
+	$audio_video_psu_file = $self->target_avi_file
+		if $job->move_final;
+	my $audio_video_psu_dir  = dirname ( $audio_video_psu_file );
+	my $audio_psu_file       = $self->audio_psu_file;
 
 	my $chunks_mask = sprintf (
 		"%s/%03d/chunks-psu-??/*",
@@ -275,36 +274,34 @@ sub get_merge_audio_command {
 	);
 
 	my $command =
-		"mkdir -m 0775 -p '$target_avi_dir' && ".
-		"avimerge -i $chunks_mask".
-		" -o $target_avi_file ".
-		" -p $audio_avi_file ".
+		"mkdir -m 0775 -p '$audio_video_psu_dir' && ".
+		"avimerge -i $avi_chunks_dir/*".
+		" -o $audio_video_psu_file ".
+		" -p $audio_psu_file ".
 		" && echo DVDRIP_SUCCESS";
 
-	$command .= " && rm $chunks_mask '$audio_avi_file'"
+	$command .= " && rm $avi_chunks_dir/* '$audio_psu_file'"
 		if $self->with_cleanup;
 
 	return $command;
 }
 
-sub get_merge_audio_command_NO_VIDEO_MERGE {
+sub get_merge_psu_command {
 	my $self = shift;
+	my %par = @_;
+	my ($psu) = @par{'psu'};
 	
-	my $job = $self->project->assigned_job or croak "No job assigned";
-
-	my $target_avi_file        = $self->target_avi_file;
-	my $target_avi_dir         = dirname ( $target_avi_file );
-	my $audio_avi_file         = $self->audio_avi_file;
-	my $merged_chunks_avi_file = $self->merged_chunks_avi_file;
+	my $target_avi_file      = $self->target_avi_file;
+	my $target_avi_dir       = dirname($target_avi_file);
+	my $audio_video_psu_dir  = $self->audio_video_psu_dir;
 
 	my $command =
 		"mkdir -m 0775 -p '$target_avi_dir' && ".
-		"avimerge -i $merged_chunks_avi_file".
-		" -o $target_avi_file ".
-		" -p $audio_avi_file ".
-		" && echo DVDRIP_SUCCESS";
+		"avimerge -o '$target_avi_file' ".
+		" -i $audio_video_psu_dir/*.avi 2>/dev/null";
 
-	$command .= " && rm '$merged_chunks_avi_file' '$audio_avi_file'"
+	$command .= " && echo DVDRIP_SUCCESS";
+	$command .= " && rm $audio_video_psu_dir/*.avi"
 		if $self->with_cleanup;
 
 	return $command;
