@@ -1,4 +1,4 @@
-# $Id: Config.pm,v 1.17 2002/06/09 09:59:28 joern Exp $
+# $Id: Config.pm,v 1.22 2002/07/15 07:27:16 joern Exp $
 
 #-----------------------------------------------------------------------
 # Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
@@ -57,9 +57,9 @@ my %CONFIG_PARAMETER = (
 	play_dvd_command => {
 		label => "DVD player command",
 		type  => 'string',
-		value => 'mplayer <dvd://%t -aid 0x8%a -chapter %c -dvdangle %m>',
+		value => 'mplayer <dvd://%t -aid %(%a+128) -chapter %c -dvdangle %m>',
 		presets => [
-			'mplayer <dvd://%t -aid 0x8%a -chapter %c -dvdangle %m>',
+			'mplayer <dvd://%t -aid %(%a+128) -chapter %c -dvdangle %m>',
 			'xine -a %a -p <dvdnav://%d:%t.%c>',
 			'xine -a %a -p <d4d://i%tt0c%(%c-1)t%(%c-1)>',
 		],
@@ -67,12 +67,21 @@ my %CONFIG_PARAMETER = (
 	play_file_command => {
 		label => "File player command",
 		type  => 'string',
-		value => 'xine -p <%f>',
+		value => 'mplayer <%f>',
 		presets => [
 			'xine -p <%f>',
 			'mplayer <%f>',
 		],
 	},
+	play_stdin_command => {
+		label => "STDIN player command",
+		type  => 'string',
+		value => 'xine stdin://mpeg2 -g -pq -a %a',
+		presets => [
+			'mplayer -aid %(%a+128) -',
+			'xine stdin://mpeg2 -g -pq -a %a',
+		],
+	},
 	base_project_dir => {
 		label => "Default Data Base Directory",
 		type => 'dir',
@@ -93,14 +102,30 @@ my %CONFIG_PARAMETER = (
 		type  => 'number',
 		value => 28646,
 	},
+	show_tooltips => {
+		label => "Show tooltips",
+		type  => 'switch',
+		value => 1,
+	},
+	default_video_codec => {
+		label => "Default Video Codec",
+		type  => 'string',
+		value => 'divx4',
+		presets => [
+			"SVCD","VCD","divx4","divx5",
+			"xvid","xvidcvs","fame",
+			"af6","opendivx"
+		],
+	},
 );
 
 my @CONFIG_ORDER = qw (
 	dvd_device base_project_dir
-	play_dvd_command play_file_command
+	play_dvd_command play_file_command play_stdin_command
+	default_video_codec show_tooltips
 	main_window_width main_window_height
 	cluster_master_local cluster_master_server
-	cluster_master_port
+	cluster_master_port  
 );
 
 sub new {
@@ -199,9 +224,25 @@ sub new {
 			tc_fast_resize  => 1,
 			tc_fast_bisection => 0,
 		),
+		Video::DVDRip::Preset->new (
+			name => "svcd_pal_16_9_4_3",
+			title => "SVCD 16:9 -> 4:3 letterbox, PAL",
+			tc_clip1_top	=> 0,
+			tc_clip1_bottom	=> 0,
+			tc_clip1_left	=> 0,
+			tc_clip1_right	=> 0,
+			tc_zoom_width	=> 480,
+			tc_zoom_height	=> 432,
+			tc_clip2_top	=> -72,
+			tc_clip2_bottom	=> -72,
+			tc_clip2_left	=> 0,
+			tc_clip2_right	=> 0,
+			tc_fast_resize  => 1,
+			tc_fast_bisection => 0,
+		),
 		Video::DVDRip::Preset->new (
 			name => "svcd_pal",
-			title => "SVCD anamorph, PAL",
+			title => "SVCD 16:9 anamorph, PAL",
 			tc_clip1_top	=> 0,
 			tc_clip1_bottom	=> 0,
 			tc_clip1_left	=> 0,
@@ -247,6 +288,22 @@ sub new {
 			tc_fast_resize  => 1,
 			tc_fast_bisection => 0,
 		),
+		Video::DVDRip::Preset->new (
+			name => "svcd_ntsc_16_9_4_3",
+			title => "SVCD 16:9 -> 4:3 letterbox, NTSC",
+			tc_clip1_top	=> 0,
+			tc_clip1_bottom	=> 0,
+			tc_clip1_left	=> 0,
+			tc_clip1_right	=> 0,
+			tc_zoom_width	=> 480,
+			tc_zoom_height	=> 432,
+			tc_clip2_top	=> -24,
+			tc_clip2_bottom	=> -24,
+			tc_clip2_left	=> 0,
+			tc_clip2_right	=> 0,
+			tc_fast_resize  => 1,
+			tc_fast_bisection => 0,
+		),
 		Video::DVDRip::Preset->new (
 			name => "svcd_ntsc",
 			title => "SVCD anamorph, NTSC",
@@ -337,7 +394,7 @@ sub save {
 	my $fh = FileHandle->new;
 
 	open ($fh, "> $filename") or confess "can't write $filename";
-	print $fh q{# $Id: Config.pm,v 1.17 2002/06/09 09:59:28 joern Exp $},"\n";
+	print $fh q{# $Id: Config.pm,v 1.22 2002/07/15 07:27:16 joern Exp $},"\n";
 	print $fh "# This file was generated by Video::DVDRip Version $Video::DVDRip::VERSION\n\n";
 
 	print $fh ${$data_sref};
